@@ -298,6 +298,8 @@ g_daemon_vfs_init (GDaemonVfs *vfs)
   
   vfs->wrapped_vfs = g_vfs_get_local ();
 
+  dbus_connection_set_exit_on_disconnect (vfs->async_bus, FALSE);
+
   _g_dbus_connection_integrate_with_main (vfs->async_bus);
 
   modules = g_io_modules_load_all_in_directory (GVFS_MODULE_DIR);
@@ -950,6 +952,13 @@ void g_vfs_uri_mapper_sftp_register (GIOModule *module);
 void
 g_io_module_load (GIOModule *module)
 {
+ /* This is so that system daemons can use gio
+  * without spawning private dbus instances.
+  * See bug 526454.
+  */
+  if (g_getenv ("DBUS_SESSION_BUS_ADDRESS") == NULL)
+    return;
+
   g_daemon_vfs_register_type (G_TYPE_MODULE (module));
   g_daemon_volume_monitor_register_types (G_TYPE_MODULE (module));
 
