@@ -94,7 +94,7 @@ struct _GVfsBackendBurn
   GMountSpec *mount_spec;
 };
 
-G_DEFINE_TYPE (GVfsBackendBurn, g_vfs_backend_burn, G_VFS_TYPE_BACKEND);
+G_DEFINE_TYPE (GVfsBackendBurn, g_vfs_backend_burn, G_VFS_TYPE_BACKEND)
 
 
 static void virtual_node_unref (VirtualNode *node);
@@ -163,7 +163,7 @@ virtual_node_unref (VirtualNode *node)
     virtual_node_free (node, TRUE);
 }
 
-static VirtualNode *
+static G_GNUC_UNUSED VirtualNode *
 virtual_node_ref (VirtualNode *node)
 {
   g_return_val_if_fail (node != NULL, NULL);
@@ -343,6 +343,7 @@ g_vfs_backend_burn_init (GVfsBackendBurn *burn_backend)
   GVfsBackend *backend = G_VFS_BACKEND (burn_backend);
   GMountSpec *mount_spec;
   
+  /* Translators: This is the name of the backend */
   g_vfs_backend_set_display_name (backend, _("Burn"));
   g_vfs_backend_set_icon_name (backend, "gnome-fs-client");
   g_vfs_backend_set_user_visible (backend, FALSE);
@@ -647,7 +648,8 @@ file_info_from_node (VirtualNode *node,
   else
     {
       g_file_info_set_name (info, "/");
-      g_file_info_set_display_name (info, _("CD Burner"));
+      /* Translators: this is the display name of the backend */
+      g_file_info_set_display_name (info, _("CD/DVD Creator"));
     }
 }
 
@@ -822,10 +824,12 @@ try_upload (GVfsBackend *backend,
   
   if (g_stat (local_path, &stat_buf) == -1)
     {
+      int errsv = errno;
+
       g_vfs_job_failed (G_VFS_JOB (job),
                         G_IO_ERROR,
-                        g_io_error_from_errno (errno),
-                        g_strerror (errno));
+                        g_io_error_from_errno (errsv),
+                        "%s", g_strerror (errsv));
       return TRUE;
     }
 
@@ -882,7 +886,7 @@ try_upload (GVfsBackend *backend,
       return TRUE;
     }
 
-  g_print ("file: %p, overwrite: %d\n", file, flags & G_FILE_COPY_OVERWRITE);
+  g_print ("file: %p, overwrite: %u\n", file, flags & G_FILE_COPY_OVERWRITE);
   if (file != NULL)
     {
       if (flags & G_FILE_COPY_OVERWRITE)
