@@ -57,6 +57,7 @@
 #define PATH_GCONF_GNOME_VFS_SMB_WORKGROUP "/system/smb/workgroup"
 
 #include <libsmbclient.h>
+#include "libsmb-compat.h"
 
 struct _GVfsBackendSmb
 {
@@ -147,7 +148,7 @@ auth_callback (SMBCCTX *context,
   char *ask_password, *ask_user, *ask_domain;
   gboolean handled, abort;
 
-  backend = smbc_option_get (context, "user_data");
+  backend = smbc_getOptionUserData (context);
 
   strncpy (password_out, "", pwmaxlen);
   
@@ -274,7 +275,7 @@ add_cached_server (SMBCCTX *context, SMBCSRV *new,
 {
   GVfsBackendSmb *backend;
 
-  backend = smbc_option_get (context, "user_data");
+  backend = smbc_getOptionUserData (context);
   
   if (backend->cached_server != NULL)
     return 1;
@@ -300,7 +301,7 @@ remove_cached_server(SMBCCTX * context, SMBCSRV * server)
 {
   GVfsBackendSmb *backend;
 
-  backend = smbc_option_get (context, "user_data");
+  backend = smbc_getOptionUserData (context);
   
   if (backend->cached_server == server)
     {
@@ -336,7 +337,7 @@ get_cached_server (SMBCCTX * context,
 {
   GVfsBackendSmb *backend;
 
-  backend = smbc_option_get (context, "user_data");
+  backend = smbc_getOptionUserData (context);
 
   if (backend->cached_server != NULL &&
       strcmp (backend->cached_server_name, server_name) == 0 &&
@@ -360,7 +361,7 @@ purge_cached (SMBCCTX * context)
 {
   GVfsBackendSmb *backend;
   
-  backend = smbc_option_get (context, "user_data");
+  backend = smbc_getOptionUserData (context);
 
   if (backend->cached_server)
     remove_cached_server(context, backend->cached_server);
@@ -457,6 +458,7 @@ do_mount (GVfsBackend *backend,
   int res;
   char *display_name;
   GMountSpec *smb_mount_spec;
+  smbc_stat_fn smbc_stat;
 
   smb_context = smbc_new_context ();
   if (smb_context == NULL)
@@ -466,37 +468,31 @@ do_mount (GVfsBackend *backend,
 			_("Internal Error (%s)"), "Failed to allocate smb context");
       return;
     }
-  smbc_option_set (smb_context, "user_data", backend);
-  
-  smb_context->debug = 0;
-  
-  smb_context->callbacks.auth_fn = NULL;
-  smbc_option_set (smb_context, "auth_function",
-		   (void *) auth_callback);
-  
-  smb_context->callbacks.add_cached_srv_fn    = add_cached_server;
-  smb_context->callbacks.get_cached_srv_fn    = get_cached_server;
-  smb_context->callbacks.remove_cached_srv_fn = remove_cached_server;
-  smb_context->callbacks.purge_cached_fn      = purge_cached;
- 
-  /* libsmbclient frees this on it's own, so make sure 
-   * to use simple system malloc */
-  if (default_workgroup != NULL)
-    smb_context->workgroup = strdup (default_workgroup);
-  
-  smb_context->flags = 0;
-  
-#if defined(HAVE_SAMBA_FLAGS) 
-#if defined(SMB_CTX_FLAG_USE_KERBEROS) && defined(SMB_CTX_FLAG_FALLBACK_AFTER_KERBEROS)
-  smb_context->flags |= SMB_CTX_FLAG_USE_KERBEROS | SMB_CTX_FLAG_FALLBACK_AFTER_KERBEROS;
-#endif
-#if defined(SMBCCTX_FLAG_NO_AUTO_ANONYMOUS_LOGON)
-  smb_context->flags |= SMBCCTX_FLAG_NO_AUTO_ANONYMOUS_LOGON;
-#endif
-#endif
+  smbc_setOptionUserData (smb_context, backend);
 
+  smbc_setDebug (smb_context, 0);
+  smbc_setFunctionAuthDataWithContext (smb_context, auth_callback);
+  
+  smbc_setFunctionAddCachedServer (smb_context, add_cached_server);
+  smbc_setFunctionGetCachedServer (smb_context, get_cached_server);
+  smbc_setFunctionRemoveCachedServer (smb_context, remove_cached_server);
+  smbc_setFunctionPurgeCachedServers (smb_context, purge_cached);
+
+  /* FIXME: is strdup() still needed here? -- removed */
+  if (default_workgroup != NULL)
+    smbc_setWorkgroup (smb_context, default_workgroup);
+
+#ifndef DEPRECATED_SMBC_INTERFACE
+  smb_context->flags = 0;
+#endif
+  
+  smbc_setOptionUseKerberos (smb_context, 1);
+  smbc_setOptionFallbackAfterKerberos (smb_context, 1);  
+  smbc_setOptionNoAutoAnonymousLogin (smb_context, 1);
+
+  
 #if 0
-  smbc_option_set(smb_context, "debug_stderr", (void *) 1);
+  smbc_setOptionDebugToStderr (smb_context, 1);
 #endif
   
   if (!smbc_init_context (smb_context))
@@ -536,8 +532,9 @@ do_mount (GVfsBackend *backend,
   do
     {
       op_backend->mount_try_again = FALSE;
-      
-      res = smb_context->stat (smb_context, uri, &st);
+
+      smbc_stat = smbc_getFunctionStat (smb_context);
+      res = smbc_stat (smb_context, uri, &st);
       
       if (res == 0 ||
 	  (errno != EACCES && errno != EPERM))
@@ -615,15 +612,33 @@ do_open_for_read (GVfsBackend *backend,
   GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
   char *uri;
   SMBCFILE *file;
+  struct stat st;
+  smbc_open_fn smbc_open;
+  smbc_stat_fn smbc_stat;
+  int res;
+  int olderr;
+
 
   uri = create_smb_uri (op_backend->server, op_backend->share, filename);
-  file = op_backend->smb_context->open (op_backend->smb_context, uri, O_RDONLY, 0);
-  g_free (uri);
+  smbc_open = smbc_getFunctionOpen (op_backend->smb_context);
+  file = smbc_open (op_backend->smb_context, uri, O_RDONLY, 0);
 
   if (file == NULL)
-    g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
+    {
+      olderr = errno;
+      smbc_stat = smbc_getFunctionStat (op_backend->smb_context);
+      res = smbc_stat (op_backend->smb_context, uri, &st);
+      g_free (uri);
+      if ((res == 0) && (S_ISDIR (st.st_mode)))
+            g_vfs_job_failed (G_VFS_JOB (job),
+                              G_IO_ERROR, G_IO_ERROR_IS_DIRECTORY,
+                             _("Can't open directory"));
+      else
+        g_vfs_job_failed_from_errno (G_VFS_JOB (job), olderr);
+  }
   else
     {
+      
       g_vfs_job_open_for_read_set_can_seek (job, TRUE);
       g_vfs_job_open_for_read_set_handle (job, file);
       g_vfs_job_succeeded (G_VFS_JOB (job));
@@ -639,6 +654,7 @@ do_read (GVfsBackend *backend,
 {
   GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
   ssize_t res;
+  smbc_read_fn smbc_read;
 
   /* For some reason requests of 65536 bytes broke for me (returned 0)
    * Maybe some smb protocol limit
@@ -646,7 +662,8 @@ do_read (GVfsBackend *backend,
   if (bytes_requested > 65535)
     bytes_requested = 65535;
   
-  res = op_backend->smb_context->read (op_backend->smb_context, (SMBCFILE *)handle, buffer, bytes_requested);
+  smbc_read = smbc_getFunctionRead (op_backend->smb_context);
+  res = smbc_read (op_backend->smb_context, (SMBCFILE *)handle, buffer, bytes_requested);
 
   if (res == -1)
     g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
@@ -668,6 +685,7 @@ do_seek_on_read (GVfsBackend *backend,
   GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
   int whence;
   off_t res;
+  smbc_lseek_fn smbc_lseek;
 
   switch (type)
     {
@@ -687,7 +705,8 @@ do_seek_on_read (GVfsBackend *backend,
       return;
     }
 
-  res = op_backend->smb_context->lseek (op_backend->smb_context, (SMBCFILE *)handle, offset, whence);
+  smbc_lseek = smbc_getFunctionLseek (op_backend->smb_context);
+  res = smbc_lseek (op_backend->smb_context, (SMBCFILE *)handle, offset, whence);
 
   if (res == (off_t)-1)
     g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
@@ -707,8 +726,10 @@ do_close_read (GVfsBackend *backend,
 {
   GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
   ssize_t res;
+  smbc_close_fn smbc_close;
 
-  res = op_backend->smb_context->close_fn (op_backend->smb_context, (SMBCFILE *)handle);
+  smbc_close = smbc_getFunctionClose (op_backend->smb_context);
+  res = smbc_close (op_backend->smb_context, (SMBCFILE *)handle);
   if (res == -1)
     g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
   else
@@ -741,9 +762,11 @@ do_create (GVfsBackend *backend,
   char *uri;
   SMBCFILE *file;
   SmbWriteHandle *handle;
+  smbc_open_fn smbc_open;
 
   uri = create_smb_uri (op_backend->server, op_backend->share, filename);
-  file = op_backend->smb_context->open (op_backend->smb_context, uri,
+  smbc_open = smbc_getFunctionOpen (op_backend->smb_context);
+  file = smbc_open (op_backend->smb_context, uri,
 					O_CREAT|O_WRONLY|O_EXCL, 0666);
   g_free (uri);
 
@@ -771,9 +794,12 @@ do_append_to (GVfsBackend *backend,
   SMBCFILE *file;
   SmbWriteHandle *handle;
   off_t initial_offset;
+  smbc_open_fn smbc_open;
+  smbc_lseek_fn smbc_lseek;
 
   uri = create_smb_uri (op_backend->server, op_backend->share, filename);
-  file = op_backend->smb_context->open (op_backend->smb_context, uri,
+  smbc_open = smbc_getFunctionOpen (op_backend->smb_context);
+  file = smbc_open (op_backend->smb_context, uri,
 					O_CREAT|O_WRONLY|O_APPEND, 0666);
   g_free (uri);
 
@@ -784,7 +810,8 @@ do_append_to (GVfsBackend *backend,
       handle = g_new0 (SmbWriteHandle, 1);
       handle->file = file;
 
-      initial_offset = op_backend->smb_context->lseek (op_backend->smb_context, file,
+      smbc_lseek = smbc_getFunctionLseek (op_backend->smb_context);
+      initial_offset = smbc_lseek (op_backend->smb_context, file,
 						       0, SEEK_CUR);
       if (initial_offset == (off_t) -1)
 	g_vfs_job_open_for_write_set_can_seek (job, FALSE);
@@ -837,6 +864,7 @@ open_tmpfile (GVfsBackendSmb *backend,
   char *dir_uri, *tmp_uri;
   char filename[] = "~gvfXXXX.tmp";
   SMBCFILE *file;
+  smbc_open_fn smbc_open;
 
   dir_uri = get_dir_from_uri (uri);
  
@@ -844,7 +872,8 @@ open_tmpfile (GVfsBackendSmb *backend,
     random_chars (filename + 4, 4);
     tmp_uri = g_strconcat (dir_uri, filename, NULL);
 
-    file = backend->smb_context->open (backend->smb_context, tmp_uri,
+    smbc_open = smbc_getFunctionOpen (backend->smb_context);
+    file = smbc_open (backend->smb_context, tmp_uri,
 				       O_CREAT|O_WRONLY|O_EXCL, 0666);
   } while (file == NULL && errno == EEXIST);
 
@@ -874,19 +903,28 @@ copy_file (GVfsBackendSmb *backend,
   ssize_t res;
   char *p;
   gboolean succeeded;
+  smbc_open_fn smbc_open;
+  smbc_read_fn smbc_read;
+  smbc_write_fn smbc_write;
+  smbc_close_fn smbc_close;
   
 
   from_file = NULL;
   to_file = NULL;
 
   succeeded = FALSE;
-  
-  from_file = backend->smb_context->open (backend->smb_context, from_uri,
+
+  smbc_open = smbc_getFunctionOpen (backend->smb_context);
+  smbc_read = smbc_getFunctionRead (backend->smb_context);
+  smbc_write = smbc_getFunctionWrite (backend->smb_context);
+  smbc_close = smbc_getFunctionClose (backend->smb_context);
+
+  from_file = smbc_open (backend->smb_context, from_uri,
 					  O_RDONLY, 0666);
   if (from_file == NULL || g_vfs_job_is_cancelled (job))
     goto out;
   
-  to_file = backend->smb_context->open (backend->smb_context, to_uri,
+  to_file = smbc_open (backend->smb_context, to_uri,
 					O_CREAT|O_WRONLY|O_TRUNC, 0666);
 
   if (from_file == NULL || g_vfs_job_is_cancelled (job))
@@ -895,7 +933,7 @@ copy_file (GVfsBackendSmb *backend,
   while (1)
     {
       
-      res = backend->smb_context->read (backend->smb_context, from_file,
+      res = smbc_read (backend->smb_context, from_file,
 					buffer, sizeof(buffer));
       if (res < 0 || g_vfs_job_is_cancelled (job))
 	goto out;
@@ -906,7 +944,7 @@ copy_file (GVfsBackendSmb *backend,
       p = buffer;
       while (buffer_size > 0)
 	{
-	  res = backend->smb_context->write (backend->smb_context, to_file,
+	  res = smbc_write (backend->smb_context, to_file,
 					     p, buffer_size);
 	  if (res < 0 || g_vfs_job_is_cancelled (job))
 	    goto out;
@@ -918,9 +956,9 @@ copy_file (GVfsBackendSmb *backend,
  
  out: 
   if (to_file)
-    backend->smb_context->close_fn (backend->smb_context, to_file);
+	  smbc_close (backend->smb_context, to_file);
   if (from_file)
-    backend->smb_context->close_fn (backend->smb_context, from_file);
+	  smbc_close (backend->smb_context, from_file);
   return succeeded;
 }
 
@@ -945,6 +983,8 @@ do_replace (GVfsBackend *backend,
   SMBCFILE *file;
   GError *error = NULL;
   SmbWriteHandle *handle;
+  smbc_open_fn smbc_open;
+  smbc_stat_fn smbc_stat;
 
   uri = create_smb_uri (op_backend->server, op_backend->share, filename);
   tmp_uri = NULL;
@@ -952,8 +992,11 @@ do_replace (GVfsBackend *backend,
     backup_uri = g_strconcat (uri, "~", NULL);
   else
     backup_uri = NULL;
+
+  smbc_open = smbc_getFunctionOpen (op_backend->smb_context);
+  smbc_stat = smbc_getFunctionStat (op_backend->smb_context);
   
-  file = op_backend->smb_context->open (op_backend->smb_context, uri,
+  file = smbc_open (op_backend->smb_context, uri,
 					O_CREAT|O_WRONLY|O_EXCL, 0);
   if (file == NULL && errno != EEXIST)
     {
@@ -968,7 +1011,7 @@ do_replace (GVfsBackend *backend,
     {
       if (etag != NULL)
 	{
-	  res = op_backend->smb_context->stat (op_backend->smb_context, uri, &original_stat);
+	  res = smbc_stat (op_backend->smb_context, uri, &original_stat);
 	  
 	  if (res == 0)
 	    {
@@ -1025,7 +1068,7 @@ do_replace (GVfsBackend *backend,
 	      backup_uri = NULL;
 	    }
 	  
-	  file = op_backend->smb_context->open (op_backend->smb_context, uri,
+	  file = smbc_open (op_backend->smb_context, uri,
 						O_CREAT|O_WRONLY|O_TRUNC, 0);
 	  if (file == NULL)
 	    {
@@ -1076,8 +1119,10 @@ do_write (GVfsBackend *backend,
   GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
   SmbWriteHandle *handle = _handle;
   ssize_t res;
+  smbc_write_fn smbc_write;
 
-  res = op_backend->smb_context->write (op_backend->smb_context, handle->file,
+  smbc_write = smbc_getFunctionWrite (op_backend->smb_context);
+  res = smbc_write (op_backend->smb_context, handle->file,
 					buffer, buffer_size);
   if (res == -1)
     g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
@@ -1099,6 +1144,7 @@ do_seek_on_write (GVfsBackend *backend,
   SmbWriteHandle *handle = _handle;
   int whence;
   off_t res;
+  smbc_lseek_fn smbc_lseek;
 
   switch (type)
     {
@@ -1118,7 +1164,8 @@ do_seek_on_write (GVfsBackend *backend,
       return;
     }
 
-  res = op_backend->smb_context->lseek (op_backend->smb_context, handle->file, offset, whence);
+  smbc_lseek = smbc_getFunctionLseek (op_backend->smb_context);
+  res = smbc_lseek (op_backend->smb_context, handle->file, offset, whence);
 
   if (res == (off_t)-1)
     g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
@@ -1141,17 +1188,26 @@ do_close_write (GVfsBackend *backend,
   struct stat stat_at_close;
   int stat_res;
   ssize_t res;
+  smbc_fstat_fn smbc_fstat;
+  smbc_close_fn smbc_close;
+  smbc_unlink_fn smbc_unlink;
+  smbc_rename_fn smbc_rename;
 
-  stat_res = op_backend->smb_context->fstat (op_backend->smb_context, handle->file, &stat_at_close);
+  smbc_fstat = smbc_getFunctionFstat (op_backend->smb_context);
+  smbc_close = smbc_getFunctionClose (op_backend->smb_context);
+  smbc_unlink = smbc_getFunctionUnlink (op_backend->smb_context);
+  smbc_rename = smbc_getFunctionRename (op_backend->smb_context);
   
-  res = op_backend->smb_context->close_fn (op_backend->smb_context, handle->file);
+  stat_res = smbc_fstat (op_backend->smb_context, handle->file, &stat_at_close);
+  
+  res = smbc_close (op_backend->smb_context, handle->file);
 
   if (res == -1)
     {
       g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
       
       if (handle->tmp_uri)
-	op_backend->smb_context->unlink (op_backend->smb_context, handle->tmp_uri);
+    	  smbc_unlink (op_backend->smb_context, handle->tmp_uri);
       goto out;
     }
 
@@ -1159,13 +1215,13 @@ do_close_write (GVfsBackend *backend,
     {
       if (handle->backup_uri)
 	{
-	  res = op_backend->smb_context->rename (op_backend->smb_context, handle->uri,
+	  res = smbc_rename (op_backend->smb_context, handle->uri,
 						 op_backend->smb_context, handle->backup_uri);
 	  if (res ==  -1)
 	    {
               int errsv = errno;
 
-	      op_backend->smb_context->unlink (op_backend->smb_context, handle->tmp_uri);
+          smbc_unlink (op_backend->smb_context, handle->tmp_uri);
 	      g_vfs_job_failed (G_VFS_JOB (job),
 				G_IO_ERROR, G_IO_ERROR_CANT_CREATE_BACKUP,
 				_("Backup file creation failed: %s"), g_strerror (errsv));
@@ -1173,13 +1229,13 @@ do_close_write (GVfsBackend *backend,
 	    }
 	}
       else
-	op_backend->smb_context->unlink (op_backend->smb_context, handle->uri);
+	smbc_unlink (op_backend->smb_context, handle->uri);
       
-      res = op_backend->smb_context->rename (op_backend->smb_context, handle->tmp_uri,
+      res = smbc_rename (op_backend->smb_context, handle->tmp_uri,
 					     op_backend->smb_context, handle->uri);
       if (res ==  -1)
 	{
-	  op_backend->smb_context->unlink (op_backend->smb_context, handle->tmp_uri);
+	  smbc_unlink (op_backend->smb_context, handle->tmp_uri);
 	  g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
 	  goto out;
 	}
@@ -1212,9 +1268,14 @@ set_info_from_stat (GVfsBackendSmb *backend,
   char *content_type;
   char *display_name;
 
-  if (basename) 
-    g_file_info_set_name (info, basename);
+  if (basename)
+    {
+      g_file_info_set_name (info, basename);
+      if (*basename == '.')
+	g_file_info_set_is_hidden (info, TRUE);
+    }
 
+  
   if (basename != NULL &&
       g_file_attribute_matcher_matches (matcher,
                                         G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME))
@@ -1295,35 +1356,9 @@ set_info_from_stat (GVfsBackendSmb *backend,
 	  
 	  if (content_type)
 	    {
-	      char *mimetype_icon, *generic_mimetype_icon, *type_icon, *p;
-	      char *icon_names[3];
-	      int i;
-
-	      mimetype_icon = g_strdup (content_type);
-	      g_strdelimit (mimetype_icon, "/", '-');
-	      
-	      p = strchr (content_type, '/');
-	      if (p == NULL)
-		p = content_type + strlen (content_type);
-	      
-	      generic_mimetype_icon = g_malloc (p - content_type + strlen ("-x-generic") + 1);
-	      memcpy (generic_mimetype_icon, content_type, p - content_type);
-	      memcpy (generic_mimetype_icon + (p - content_type), "-x-generic", strlen ("-x-generic"));
-	      generic_mimetype_icon[(p - content_type) + strlen ("-x-generic")] = 0;
-	      
-	      type_icon = "text-x-generic";
-	      
-	      i = 0;
-	      icon_names[i++] = mimetype_icon;
-	      icon_names[i++] = generic_mimetype_icon;
-	      if (strcmp (generic_mimetype_icon, type_icon) != 0 &&
-		  strcmp (mimetype_icon, type_icon) != 0) 
-		icon_names[i++] = type_icon;
-	      
-	      icon = g_themed_icon_new_from_names (icon_names, i);
-	      
-	      g_free (mimetype_icon);
-	      g_free (generic_mimetype_icon);
+              icon = g_content_type_get_icon (content_type);
+              if (G_IS_THEMED_ICON (icon))
+                g_themed_icon_append_name (G_THEMED_ICON (icon), "text-x-generic");
 	    }
 	}
       
@@ -1332,12 +1367,12 @@ set_info_from_stat (GVfsBackendSmb *backend,
 	  g_file_info_set_content_type (info, content_type);
 	  g_free (content_type);
 	}
+
+      if (icon == NULL)
+	icon = g_themed_icon_new ("text-x-generic");
       
-      if (icon)
-	{
-	  g_file_info_set_icon (info, icon);
-	  g_object_unref (icon);
-	}
+      g_file_info_set_icon (info, icon);
+      g_object_unref (icon);
   }
   
   /* Don't trust n_link, uid, gid, etc returned from libsmb, its just made up.
@@ -1398,9 +1433,11 @@ do_query_info (GVfsBackend *backend,
   char *uri;
   int res, saved_errno;
   char *basename;
+  smbc_stat_fn smbc_stat;
 
   uri = create_smb_uri (op_backend->server, op_backend->share, filename);
-  res = op_backend->smb_context->stat (op_backend->smb_context, uri, &st);
+  smbc_stat = smbc_getFunctionStat (op_backend->smb_context);
+  res = smbc_stat (op_backend->smb_context, uri, &st);
   saved_errno = errno;
   g_free (uri);
 
@@ -1471,9 +1508,19 @@ do_enumerate (GVfsBackend *backend,
   GFileInfo *info;
   GString *uri;
   int uri_start_len;
+  smbc_opendir_fn smbc_opendir;
+  smbc_getdents_fn smbc_getdents;
+  smbc_stat_fn smbc_stat;
+  smbc_closedir_fn smbc_closedir;
 
   uri = create_smb_uri_string (op_backend->server, op_backend->share, filename);
-  dir = op_backend->smb_context->opendir (op_backend->smb_context, uri->str);
+  
+  smbc_opendir = smbc_getFunctionOpendir (op_backend->smb_context);
+  smbc_getdents = smbc_getFunctionGetdents (op_backend->smb_context);
+  smbc_stat = smbc_getFunctionStat (op_backend->smb_context);
+  smbc_closedir = smbc_getFunctionClosedir (op_backend->smb_context);
+  
+  dir = smbc_opendir (op_backend->smb_context, uri->str);
 
   if (dir == NULL)
     {
@@ -1496,7 +1543,7 @@ do_enumerate (GVfsBackend *backend,
     {
       files = NULL;
       
-      res = op_backend->smb_context->getdents (op_backend->smb_context, dir, (struct smbc_dirent *)dirents, sizeof (dirents));
+      res = smbc_getdents (op_backend->smb_context, dir, (struct smbc_dirent *)dirents, sizeof (dirents));
       if (res <= 0)
 	break;
       
@@ -1528,7 +1575,7 @@ do_enumerate (GVfsBackend *backend,
 		}
 	      else
 		{
-		  stat_res = op_backend->smb_context->stat (op_backend->smb_context,
+		  stat_res = smbc_stat (op_backend->smb_context,
 							    uri->str, &st);
 		  if (stat_res == 0)
 		    {
@@ -1553,7 +1600,7 @@ do_enumerate (GVfsBackend *backend,
 	}
     }
       
-  res = op_backend->smb_context->closedir (op_backend->smb_context, dir);
+  res = smbc_closedir (op_backend->smb_context, dir);
 
   g_vfs_job_enumerate_done (job);
 
@@ -1576,6 +1623,7 @@ do_set_display_name (GVfsBackend *backend,
   char *from_uri, *to_uri;
   char *dirname, *new_path;
   int res, errsv;
+  smbc_rename_fn smbc_rename;
 
   dirname = g_path_get_dirname (filename);
 
@@ -1589,7 +1637,8 @@ do_set_display_name (GVfsBackend *backend,
   from_uri = create_smb_uri (op_backend->server, op_backend->share, filename);
   to_uri = create_smb_uri (op_backend->server, op_backend->share, new_path);
   
-  res = op_backend->smb_context->rename (op_backend->smb_context, from_uri,
+  smbc_rename = smbc_getFunctionRename (op_backend->smb_context);
+  res = smbc_rename (op_backend->smb_context, from_uri,
 					 op_backend->smb_context, to_uri);
   errsv = errno;
   g_free (from_uri);
@@ -1614,10 +1663,18 @@ do_delete (GVfsBackend *backend,
   struct stat statbuf;
   char *uri;
   int errsv, res;
+  smbc_stat_fn smbc_stat;
+  smbc_rmdir_fn smbc_rmdir;
+  smbc_unlink_fn smbc_unlink;
+
 
   uri = create_smb_uri (op_backend->server, op_backend->share, filename);
 
-  res = op_backend->smb_context->stat (op_backend->smb_context, uri, &statbuf);
+  smbc_stat = smbc_getFunctionStat (op_backend->smb_context);
+  smbc_rmdir = smbc_getFunctionRmdir (op_backend->smb_context);
+  smbc_unlink = smbc_getFunctionUnlink (op_backend->smb_context);
+
+  res = smbc_stat (op_backend->smb_context, uri, &statbuf);
   if (res == -1)
     {
       errsv = errno;
@@ -1632,9 +1689,9 @@ do_delete (GVfsBackend *backend,
     }
 
   if (S_ISDIR (statbuf.st_mode))
-    res = op_backend->smb_context->rmdir (op_backend->smb_context, uri);
+    res = smbc_rmdir (op_backend->smb_context, uri);
   else
-    res = op_backend->smb_context->unlink (op_backend->smb_context, uri);
+    res = smbc_unlink (op_backend->smb_context, uri);
   errsv = errno;
   g_free (uri);
 
@@ -1652,9 +1709,11 @@ do_make_directory (GVfsBackend *backend,
   GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
   char *uri;
   int errsv, res;
+  smbc_mkdir_fn smbc_mkdir;
 
   uri = create_smb_uri (op_backend->server, op_backend->share, filename);
-  res = op_backend->smb_context->mkdir (op_backend->smb_context, uri, 0666);
+  smbc_mkdir = smbc_getFunctionMkdir (op_backend->smb_context);
+  res = smbc_mkdir (op_backend->smb_context, uri, 0666);
   errsv = errno;
   g_free (uri);
 
@@ -1678,10 +1737,18 @@ do_move (GVfsBackend *backend,
   gboolean destination_exist, source_is_dir;
   struct stat statbuf;
   int res, errsv;
+  smbc_stat_fn smbc_stat;
+  smbc_rename_fn smbc_rename;
+  smbc_unlink_fn smbc_unlink;
+
   
   source_uri = create_smb_uri (op_backend->server, op_backend->share, source);
 
-  res = op_backend->smb_context->stat (op_backend->smb_context, source_uri, &statbuf);
+  smbc_stat = smbc_getFunctionStat (op_backend->smb_context);
+  smbc_rename = smbc_getFunctionRename (op_backend->smb_context);
+  smbc_unlink = smbc_getFunctionUnlink (op_backend->smb_context);
+
+  res = smbc_stat (op_backend->smb_context, source_uri, &statbuf);
   if (res == -1)
     {
       errsv = errno;
@@ -1700,7 +1767,7 @@ do_move (GVfsBackend *backend,
   dest_uri = create_smb_uri (op_backend->server, op_backend->share, destination);
   
   destination_exist = FALSE;
-  res = op_backend->smb_context->stat (op_backend->smb_context, dest_uri, &statbuf);
+  res = smbc_stat (op_backend->smb_context, dest_uri, &statbuf);
   if (res == 0)
     {
       destination_exist = TRUE; /* Target file exists */
@@ -1734,7 +1801,7 @@ do_move (GVfsBackend *backend,
   if (flags & G_FILE_COPY_BACKUP && destination_exist)
     {
       backup_uri = g_strconcat (dest_uri, "~", NULL);
-      res = op_backend->smb_context->rename (op_backend->smb_context, dest_uri,
+      res = smbc_rename (op_backend->smb_context, dest_uri,
 					     op_backend->smb_context, backup_uri);
       if (res == -1)
 	{
@@ -1755,7 +1822,7 @@ do_move (GVfsBackend *backend,
     {
       /* Source is a dir, destination exists (and is not a dir, because that would have failed
 	 earlier), and we're overwriting. Manually remove the target so we can do the rename. */
-      res = op_backend->smb_context->unlink (op_backend->smb_context, dest_uri);
+      res = smbc_unlink (op_backend->smb_context, dest_uri);
       errsv = errno;
       if (res == -1)
 	{
@@ -1770,7 +1837,7 @@ do_move (GVfsBackend *backend,
     }
 
   
-  res = op_backend->smb_context->rename (op_backend->smb_context, source_uri,
+  res = smbc_rename (op_backend->smb_context, source_uri,
 					 op_backend->smb_context, dest_uri);
   errsv = errno;
   g_free (source_uri);
