@@ -33,7 +33,7 @@ static void soup_input_stream_seekable_iface_init (GSeekableIface *seekable_ifac
 
 G_DEFINE_TYPE_WITH_CODE (SoupInputStream, soup_input_stream, G_TYPE_INPUT_STREAM,
 			 G_IMPLEMENT_INTERFACE (G_TYPE_SEEKABLE,
-						soup_input_stream_seekable_iface_init));
+						soup_input_stream_seekable_iface_init))
 
 typedef void (*SoupInputStreamCallback) (GInputStream *);
 
@@ -121,9 +121,7 @@ soup_input_stream_finalize (GObject *object)
   g_signal_handlers_disconnect_by_func (priv->msg, G_CALLBACK (soup_input_stream_got_chunk), stream);
   g_signal_handlers_disconnect_by_func (priv->msg, G_CALLBACK (soup_input_stream_finished), stream);
   g_object_unref (priv->msg);
-
-  if (priv->leftover_buffer)
-    g_free (priv->leftover_buffer);
+  g_free (priv->leftover_buffer);
 
   if (G_OBJECT_CLASS (soup_input_stream_parent_class)->finalize)
     (*G_OBJECT_CLASS (soup_input_stream_parent_class)->finalize) (object);
@@ -207,7 +205,6 @@ soup_input_stream_new (SoupSession *session, SoupMessage *msg)
   SoupInputStream *stream;
   SoupInputStreamPrivate *priv;
 
-  g_return_val_if_fail (SOUP_IS_SESSION_ASYNC (session), NULL);
   g_return_val_if_fail (SOUP_IS_MESSAGE (msg), NULL);
 
   stream = g_object_new (SOUP_TYPE_INPUT_STREAM, NULL);
@@ -460,11 +457,9 @@ soup_input_stream_send (GInputStream *stream,
 			GCancellable *cancellable,
 			GError      **error)
 {
-  SoupInputStreamPrivate *priv;
   gboolean result;
 
   g_return_val_if_fail (SOUP_IS_INPUT_STREAM (stream), FALSE);
-  priv = SOUP_INPUT_STREAM_GET_PRIVATE (stream);
 
   if (!g_input_stream_set_pending (stream, error))
       return FALSE;
@@ -574,6 +569,10 @@ send_async_finished (GInputStream *stream)
 {
   SoupInputStreamPrivate *priv = SOUP_INPUT_STREAM_GET_PRIVATE (stream);
   GSimpleAsyncResult *result;
+  GError *error = NULL;
+
+  if (!g_cancellable_set_error_if_cancelled (priv->cancellable, &error))
+    set_error_if_http_failed (priv->msg, &error);
 
   priv->got_headers_cb = NULL;
   priv->finished_cb = NULL;
@@ -581,7 +580,13 @@ send_async_finished (GInputStream *stream)
 
   result = priv->result;
   priv->result = NULL;
-  g_simple_async_result_set_op_res_gboolean (result, SOUP_STATUS_IS_SUCCESSFUL (priv->msg->status_code));
+
+  g_simple_async_result_set_op_res_gboolean (result, error == NULL);
+  if (error)
+    {
+      g_simple_async_result_set_from_error (result, error);
+      g_error_free (error);
+    }
   g_simple_async_result_complete (result);
 }
 
@@ -637,11 +642,9 @@ soup_input_stream_send_async (GInputStream        *stream,
 			      GAsyncReadyCallback  callback,
 			      gpointer             user_data)
 {
-  SoupInputStreamPrivate *priv = SOUP_INPUT_STREAM_GET_PRIVATE (stream);
   GError *error = NULL;
 
   g_return_if_fail (SOUP_IS_INPUT_STREAM (stream));
-  priv = SOUP_INPUT_STREAM_GET_PRIVATE (stream);
 
   if (!g_input_stream_set_pending (stream, &error))
     {
@@ -677,8 +680,12 @@ soup_input_stream_send_finish (GInputStream  *stream,
 
   g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (result), FALSE);
   simple = G_SIMPLE_ASYNC_RESULT (result);
+
   g_return_val_if_fail (g_simple_async_result_get_source_tag (simple) == soup_input_stream_send_async, FALSE);
 
+  if (g_simple_async_result_propagate_error (simple, error))
+    return FALSE;
+
   return g_simple_async_result_get_op_res_gboolean (simple);
 }
 
@@ -835,6 +842,18 @@ soup_input_stream_seek (GSeekable     *seekable,
   SoupInputStreamPrivate *priv = SOUP_INPUT_STREAM_GET_PRIVATE (seekable);
   char *range;
 
+  if (type == G_SEEK_END)
+    {
+      /* FIXME: we could send "bytes=-offset", but unless we know the
+       * Content-Length, we wouldn't be able to answer a tell() properly.
+       * We could find the Content-Length by doing a HEAD...
+       */
+
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                   "G_SEEK_END not currently supported");
+      return FALSE;
+    }
+
   if (!g_input_stream_set_pending (stream, error))
       return FALSE;
 
@@ -853,12 +872,8 @@ soup_input_stream_seek (GSeekable     *seekable,
       break;
 
     case G_SEEK_END:
-      /* FIXME: we could send "bytes=-offset", but unless we know the
-       * Content-Length, we wouldn't be able to answer a tell() properly.
-       * We could find the Content-Length by doing a HEAD...
-       */
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-		   "G_SEEK_END not currently supported");
+      range = NULL; /* keep compilers happy */
+      g_return_val_if_reached (FALSE);
       break;
 
     default:
