@@ -30,6 +30,7 @@ struct _HalDevicePrivate
   LibHalContext *hal_ctx;
   LibHalPropertySet *properties;
   char *udi;
+  GTimeVal time_added;
 };
 
 enum {
@@ -40,7 +41,7 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_DYNAMIC_TYPE (HalDevice, hal_device, G_TYPE_OBJECT);
+G_DEFINE_DYNAMIC_TYPE (HalDevice, hal_device, G_TYPE_OBJECT)
 
 static void
 hal_device_finalize (HalDevice *device)
@@ -91,6 +92,7 @@ static void
 hal_device_init (HalDevice *device)
 {
   device->priv = g_new0 (HalDevicePrivate, 1);
+  g_get_current_time (&(device->priv->time_added));
 }
 
 const char *
@@ -243,7 +245,7 @@ hal_device_new_from_udi_and_properties (LibHalContext *hal_ctx,
   HalDevice *device;
   
   device = HAL_DEVICE (g_object_new (HAL_TYPE_DEVICE, NULL));
-  device->priv->udi = udi;
+  device->priv->udi = g_strdup (udi);
   device->priv->hal_ctx = hal_ctx;
   device->priv->properties = properties;
   return device;
@@ -292,3 +294,17 @@ hal_device_register (GIOModule *module)
 {
   hal_device_register_type (G_TYPE_MODULE (module));
 }
+
+gboolean
+hal_device_is_recently_plugged_in (HalDevice *device)
+{
+  GTimeVal now;
+  glong delta_msec;
+
+  g_get_current_time (&now);
+
+  delta_msec = (now.tv_sec - device->priv->time_added.tv_sec) * 1000  + 
+    (now.tv_usec - device->priv->time_added.tv_usec) / 1000;
+
+  return delta_msec < 2000;
+}
