@@ -67,6 +67,7 @@ static gint benchmark_run (gint argc, gchar *argv []);
 GList    *benchmark_data_plots = NULL;
 gboolean  benchmark_is_running = FALSE;
 
+#if 0
 static void
 benchmark_begin_data_plot (const gchar *name, const gchar *x_unit, const gchar *y_unit)
 {
@@ -123,6 +124,8 @@ benchmark_add_data_point (gdouble x, gdouble y)
   g_array_append_val (data_set->points, data_point);
 }
 
+#endif
+
 static void
 benchmark_end (void)
 {
@@ -175,7 +178,7 @@ benchmark_quit_main_loop (void)
 }
 
 static void
-benchmark_timeout (void)
+benchmark_timeout (int signal)
 {
   benchmark_is_running = FALSE;
 }
